package ru.landgrafhomyak.db.sql_skeleton_0.api.queries

import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.sql_skeleton_0.api.misc.Column
import ru.landgrafhomyak.db.sql_skeleton_0.api.table.Table
import ru.landgrafhomyak.db.sql_skeleton_0.api.misc.DatabaseType
import ru.landgrafhomyak.db.sql_skeleton_0.api.misc.QueryParam
import ru.landgrafhomyak.db.sql_skeleton_0.api.misc._Selectable
import ru.landgrafhomyak.db.sql_skeleton_0.api.runtime.InputRow
import ru.landgrafhomyak.db.sql_skeleton_0.api.table.RowId

public interface UpdateQuery<out qUE : Any> : _Query<qUE> {
	public interface SingleRow<out qUE : Any> : UpdateQuery<qUE>, _Query.Params2Table<qUE> {
		public interface Constructor<qUE : Any, ttUE : Any> : _Query.Constructor<SingleRow<qUE>> {
			override fun createQuery(internalQueries: _Query.Constructor.Scope, outQueries: _Query.Constructor.Scope): SingleRow<qUE> =
				outQueries.updateSingleRow(this)

			@Suppress("INAPPLICABLE_JVM_NAME")
			@get:JvmName("targetTable")
			public val targetTable: Table<ttUE, *>
			public fun createUpdateSingleWithReturning(context: Scope<qUE, ttUE>): qUE

			public interface Scope<qUE : Any, ttUE : Any> {
				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("tt")
				public val tt: Table<ttUE, *>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("types")
				public val types: DatabaseType.Provider
				public fun <RT, DT : DatabaseType<RT>> queryParam(name: String, type: DT): QueryParam<RT, DT, qUE>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("rowToUpdate")
				public val rowToUpdate: QueryParam<RowId<ttUE>, DatabaseType.ROW_ID<ttUE>, qUE>

				public fun <RT, DT : DatabaseType<RT>> updateColumnWithOldValue(dst: Column<RT, DT, ttUE>, src: Column<RT, DT, ttUE>)
				public fun <RT, DT : DatabaseType<RT>> updateColumnWithParam(dst: Column<RT, DT, ttUE>, src: QueryParam<RT, DT, qUE>)

				public fun <RT, DT : DatabaseType<RT>> returnOldValue(name: String, column: Column<RT, DT, ttUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnNewValue(name: String, column: Column<RT, DT, ttUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnParam(name: String, param: QueryParam<RT, DT, ttUE>): Column<RT, DT, qUE>

				public fun ifNoRowSkip()
				public fun ifNoRowJustReturn()
				public fun ifNoRowThrow()
			}
		}
	}

	public interface FromSubquery<out qUE : Any> : UpdateQuery<qUE>, _Query.Params2Table<qUE> {
		public interface Constructor<qUE : Any, ttUE : Any, sUE : Any> : _Query.Constructor<FromSubquery<qUE>> {
			override fun createQuery(internalQueries: _Query.Constructor.Scope, outQueries: _Query.Constructor.Scope): FromSubquery<qUE> =
				outQueries.updateFromSubquery(this)

			@Suppress("INAPPLICABLE_JVM_NAME")
			@get:JvmName("targetTable")
			public val targetTable: Table<ttUE, *>

			@Suppress("INAPPLICABLE_JVM_NAME")
			@get:JvmName("selector")
			public val selector: _Selectable<sUE>

			public fun createUpdateFromSubqueryWithReturning(context: Scope<qUE, ttUE, sUE>): qUE

			public interface Scope<qUE : Any, ttUE : Any, sUE : Any> {
				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("tt")
				public val tt: Table<ttUE, *>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("src")
				public val src: _Selectable<sUE>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("types")
				public val types: DatabaseType.Provider
				public fun <RT, DT : DatabaseType<RT>> queryParam(name: String, type: DT): QueryParam<RT, DT, qUE>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("selectorParams")
				public val selectorParams: InputRow.WithRedirect<sUE, qUE>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("_getSelectorColumn")
				@set:JvmName("selectorColumn")
				public var selectorColumn: Column<RowId<ttUE>, DatabaseType.ROW_ID<ttUE>, sUE>

				public fun <RT, DT : DatabaseType<RT>> updateColumnWithSubquery(dst: Column<RT, DT, ttUE>, src: Column<RT, DT, sUE>)
				public fun <RT, DT : DatabaseType<RT>> updateColumnWithOldValue(dst: Column<RT, DT, ttUE>, src: Column<RT, DT, ttUE>)
				public fun <RT, DT : DatabaseType<RT>> updateColumnWithParam(dst: Column<RT, DT, ttUE>, src: QueryParam<RT, DT, qUE>)

				public fun <RT, DT : DatabaseType<RT>> returnFromSubquery(name: String, column: Column<RT, DT, sUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnOldValue(name: String, column: Column<RT, DT, ttUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnNewValue(name: String, column: Column<RT, DT, ttUE>): Column<RT, DT, qUE>
				public fun <RT, DT : DatabaseType<RT>> returnParam(name: String, param: QueryParam<RT, DT, ttUE>): Column<RT, DT, qUE>

				public fun ifNoRowSkip()
				public fun ifNoRowJustReturn()
				public fun ifNoRowThrow()
			}
		}
	}

	public interface MultipleRows<out qUE : Any> : UpdateQuery<qUE>, _Query.Table2Void<qUE> {
		public interface Constructor<qUE : Any, ttUE : Any> : _Query.Constructor<MultipleRows<qUE>> {
			override fun createQuery(internalQueries: _Query.Constructor.Scope, outQueries: _Query.Constructor.Scope): MultipleRows<qUE> =
				outQueries.updateMultipleRows(this)

			@Suppress("INAPPLICABLE_JVM_NAME")
			@get:JvmName("targetTable")
			public val targetTable: Table<ttUE, *>
			public fun createUpdateMultipleRows(context: Scope<qUE, ttUE>): qUE

			public interface Scope<qUE : Any, ttUE : Any> {
				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("tt")
				public val tt: Table<ttUE, *>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("types")
				public val types: DatabaseType.Provider
				public fun <RT, DT : DatabaseType<RT>> inputColumn(name: String, type: DT): QueryParam<RT, DT, qUE>

				@Suppress("INAPPLICABLE_JVM_NAME")
				@get:JvmName("rowToUpdate")
				public val rowToUpdate: QueryParam<RowId<ttUE>, DatabaseType.ROW_ID<ttUE>, qUE>

				public fun <RT, DT : DatabaseType<RT>> updateColumnWithOldValue(dst: Column<RT, DT, ttUE>, src: Column<RT, DT, ttUE>)
				public fun <RT, DT : DatabaseType<RT>> updateColumnWithParam(dst: Column<RT, DT, ttUE>, src: QueryParam<RT, DT, qUE>)

				public fun ifNoRowSkip()
				public fun ifNoRowThrow()
			}
		}
	}
}